package com.yeejoin.amos.api.tool.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.api.tool.face.model.DataBaseLinkModel;
import com.yeejoin.amos.api.tool.face.service.DataBaseLinkService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;

import java.util.List;


/**
 * @author r
 */
@RestController
@Api(tags = "数据库信息")
@RequestMapping(value = "/database_join_record")
public class DataBaseLinkResource {

    @Autowired
    private DataBaseLinkService dataBaseLinkService;

    /**
     * 数据库初始化
     * @param dataBaseLinkModel 数据库连接信息
     * @return List<DataBaseLinkModel>
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
    @PostMapping(value = "/refresh")
    @ApiOperation(httpMethod = "POST", value = "数据库初始化", notes = "数据库初始化")
    public ResponseModel<List<DataBaseLinkModel>> save(@RequestBody DataBaseLinkModel dataBaseLinkModel) throws Exception {
            return ResponseHelper.buildResponse(
                    dataBaseLinkService.refreshDataBase(dataBaseLinkService.encryptPwd(dataBaseLinkModel)));
    }

    /**
     * 列表分页查询
     * @param current 当前分页
     * @param size 分页大小
     * @return Page<DataBaseLinkModel>
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
    @GetMapping(value = "/page")
    @ApiOperation(httpMethod = "GET", value = "分页查询", notes = "分页查询")
    public ResponseModel<Page<DataBaseLinkModel>> queryForPage(
            @RequestParam(value = "current") int current,
            @RequestParam(value = "size") int size) {
        Page<DataBaseLinkModel> page = new Page<DataBaseLinkModel>();
        page.setCurrent(current);
        page.setSize(size);
        return ResponseHelper.buildResponse(dataBaseLinkService.queryForDataBasePage(page));
    }

    /**
     * 列表全部数据查询
     * @return List<DataBaseLinkModel>
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
    @ApiOperation(httpMethod = "GET", value = "列表全部数据查询", notes = "列表全部数据查询")
    @GetMapping(value = "/list")
    public ResponseModel<List<DataBaseLinkModel>> selectForList() {
        return ResponseHelper.buildResponse(dataBaseLinkService.queryForDataBaseList());
    }
}
