package com.yeejoin.amos.api.tool.face.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.api.tool.face.model.DataBaseLinkModel;
import com.yeejoin.amos.api.tool.face.orm.dao.DataBaseLinkMapper;
import com.yeejoin.amos.api.tool.face.orm.entity.DataBaseLink;
import com.yeejoin.amos.api.tool.utils.AESUtils;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.typroject.tyboot.core.rdbms.service.BaseService;

import java.sql.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 服务实现类
 *
 * @author r
 */


@Service
public class DataBaseLinkService extends BaseService<DataBaseLinkModel, DataBaseLink, DataBaseLinkMapper> {
    @Autowired
    private DataBaseLinkMapper dataBaseLinkMapper;

    /**
     * 分页查询
     */
    public Page<DataBaseLinkModel> queryForDataBasePage(Page<DataBaseLinkModel> page) {
        return this.queryForPage(page, null, false);
    }

    /**
     * 列表查询
     */
    public List<DataBaseLinkModel> queryForDataBaseList() {
        return this.queryForList("", false);
    }

    /**
     * 获取数据库名称
     */
    public String[] getDataBaseName(Connection connection) throws SQLException {
        Statement stmt = connection.createStatement();
        ResultSet rs = stmt.executeQuery("SHOW DATABASES");
        String[] result = new String[1000];
        int count = 0;
        while (rs.next()) {
            result[count++] = rs.getString(1);
        }
        return result;
    }

    /**
     * 初始化数据库连接表
     */
    public List<DataBaseLinkModel> refreshDataBase(DataBaseLinkModel dataBaseLinkModel) throws Exception {
        //清空表单
        this.remove(new QueryWrapper<>());
        //连接数据库
        Connection connection = this.connectNewDatabase(dataBaseLinkModel);

        if (connection == null)
            return null;
        List<DataBaseLinkModel> dataBaseLinkModels = new ArrayList<>();
        Collection<DataBaseLink> dataBaseLinks = new ArrayList<>();
        for (String name : this.getDataBaseName(connection)) {
            if (name == null)
                break;
            DataBaseLinkModel model = new DataBaseLinkModel();
            BeanUtils.copyProperties(model, dataBaseLinkModel);
            model.setDbName(name);
            model.setDataName(name);
            dataBaseLinkModels.add(model);
            dataBaseLinks.add(this.prepareEntity(model));
        }
        this.saveBatch(dataBaseLinks);
        connection.close();
        return dataBaseLinkModels;
    }

    /**
     * 连接数据库
     */
    public Connection connectNewDatabase(DataBaseLinkModel model) throws Exception {
        // 加载数据库驱动
        Class.forName("com.mysql.cj.jdbc.Driver");
        // 连接新增的数据库
        String dbUrl = "jdbc:mysql://" + model.getIp() + ":" + model.getPort() + "/" + model.getDbName();
        // 解密pwd，并连接
        return DriverManager.getConnection(dbUrl, model.getUserName(),AESUtils.decrypt(model.getPassWord()));
    }

    /**
     * 加密model里的密码
     */
    public DataBaseLinkModel encryptPwd(DataBaseLinkModel model) throws Exception {
            model.setPassWord(AESUtils.encrypt(model.getPassWord()));
            return model;
    }

}
