package com.yeejoin.amos.api.tool.face.service;

import com.alibaba.fastjson.JSON;
import com.yeejoin.amos.api.tool.face.model.DataBaseLinkModel;
import com.yeejoin.amos.api.tool.face.model.RelationModel;
import com.yeejoin.amos.api.tool.face.model.RelationTreeModel;
import com.yeejoin.amos.api.tool.face.orm.dao.RelationMapper;
import com.yeejoin.amos.api.tool.face.orm.entity.Relation;
import com.yeejoin.amos.api.tool.face.orm.entity.TableColumn;
import io.github.classgraph.json.JSONUtils;
import org.apache.commons.beanutils.ConvertUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.typroject.tyboot.core.foundation.utils.StringUtil;
import org.typroject.tyboot.core.rdbms.service.BaseService;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class RelationService extends BaseService<RelationModel, Relation, RelationMapper> {

    /**
     * 数据库备份路径
     */
    public static final String BACKUP_PATH = "/db/";

    /**
     * 数据库备份文本前缀
     */
    public static String ONESQL_PREFIX = "";
    private static String SUFFIX = "sql";
    private static String BR = "\r\n";
    private static String SLASH = "/";
    private static String BRANCH = ";";
    private static String SPLIT = "`";
    private static String SPACE = " ";
    private static String INSERT_INTO = " INSERT INTO ";
    private static String CREATE_INTO = " CREATE TABLE ";
    private static String VALUES = "VALUES";
    private static String LEFTBRACE = "(";
    private static String RIGHTBRACE = ")";
    private static String QUOTES = "'";
    private static String COMMA = ",";
    private static String DISABLEFOREIGN = "SET FOREIGN_KEY_CHECKS = 0;\r\n";
    private static String ABLEFOREIGN = "SET FOREIGN_KEY_CHECKS = 1;\r\n";
    private static String DELIMITER = "###################################";

    @Autowired
    private RelationMapper relationMapper;
    @Autowired
    private DataBaseLinkService dataBaseLinkService;
    @Autowired
    private AgencyTreeService agencyTreeService;
    @Autowired
    JdbcTemplate jdbcTemplate;
    @Autowired
    StudioResourceService studioResourceService;

    /**
     * 根据机构删除
     */
    public void deleteAgency(Long agencyCodeId, String sequenceNbrs) throws Exception {
        for (Long seq : (Long[]) ConvertUtils.convert(StringUtil.string2Array(sequenceNbrs),Long.class)) {
            DataBaseLinkModel model = dataBaseLinkService.queryBySeq(seq);
            Connection connection = dataBaseLinkService.connectNewDatabase(model);
            this.deleteDataBase(
                    connection, agencyTreeService.queryForAgencyCodeById(agencyCodeId), model.getDbName());
        }
    }

    /**
     * 删除数据库
     */
    public String deleteDataBase(Connection connection, String agencyCode, String dbName) throws SQLException {
        try {
            if (!connection.isClosed())
                System.out.println("Succeeded connecting to the Database!");
            //通过连接获取relation表树形结构
            List<RelationTreeModel> trees = this.createRelationTree(dbName);
            for (RelationTreeModel tree : trees) {
                //根据根节点获得根节点相关列表
                List<RelationTreeModel> array = this.createList(tree);
                //根据列表获取sql语句
                System.out.println(this.createSqlByArray(array,agencyCode));
                System.out.println(statementSql(this.createSqlByArray(array, agencyCode).toString(),connection));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        connection.close();
        return null;
    }

    /**
     * 执行sql
     */
    public int statementSql(String sql, Connection connection) {
        int count = 0;
        try {
            Statement statement = connection.createStatement();
            count = statement.executeUpdate(sql);
            statement.close();
            return count;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return count;
    }

    /**
     * 获取关系表转化为map
     */
    public List<HashMap<String, String>> queryRelation(String dbName) throws SQLException {
        StringBuilder sql = new StringBuilder("SELECT * FROM ");
        sql.append(dbName).append("_relation;");
        List<HashMap<String,String>> maps = relationMapper.getPublicItems(sql.toString());
        return maps;
    }

    /**
     * 递归建立树形结构 （算法）
     */
    public List<RelationTreeModel> createTree(List<RelationTreeModel> relationTreeModels, String parentName) {
        List<RelationTreeModel> tree = new ArrayList<>();
        for (RelationTreeModel relationTreeModel : relationTreeModels) {
            if (parentName.equals(relationTreeModel.getParentName())) {
                relationTreeModel.setChildren(createTree(relationTreeModels, relationTreeModel.getName()));
                tree.add(relationTreeModel);
            }
        }
        return tree;
    }

    /**
     * 创建节点列表
     */
    public List<RelationTreeModel> createNodes(List<HashMap<String, String>> maps) {
        List<RelationTreeModel> relationTreeModels = new ArrayList<>();
        for (HashMap<String, String> map : maps) {
            RelationTreeModel model = new RelationTreeModel(
                    map.get("TABLE2"), map.get("TABLE2_FIELD"), map.get("TABLE1_FIELD"));
            if (map.get("TABLE1") == null)
                model.setParentName("");
            else
                model.setParentName(map.get("TABLE1"));
            relationTreeModels.add(model);
        }
        return relationTreeModels;
    }

    /**
     * 创建关系表树
     */
    public List<RelationTreeModel> createRelationTree(String dbName) throws SQLException {
        List<HashMap<String, String>> maps = this.queryRelation(dbName);
        List<RelationTreeModel> trees = new ArrayList<>();
        trees.addAll(createTree(createNodes(maps), ""));
        return trees;
    }

    /**
     * 根据列表生成sql语句
     */
    public StringBuilder createSqlByArray(List<RelationTreeModel> array, String agencyCode) {
        StringBuilder sql = new StringBuilder();
        if (array.size() == 1) {
            sql.append("DELETE FROM ").append(array.get(0).getName()).append(" WHERE ");
            sql.append(array.get(0).getName()).append(".").append(array.get(0).getField());
            sql.append(" = '").append(agencyCode).append("' ;");
        } else {
            sql.append("DELETE ").append(array.get(0).getName());
            for (RelationTreeModel model : array) {
                if (!model.getParentName().equals(""))
                    sql.append(", ").append(model.getName());
            }
            sql.append(" FROM ").append(array.get(0).getName());
            for (RelationTreeModel model : array) {
                if (!model.getParentName().equals("")) {
                    sql.append(" LEFT JOIN ").append(model.getName()).append(" ON ");
                    sql.append(model.getParentName()).append(".").append(model.getParentField()).append(" = ");
                    sql.append(model.getName()).append(".").append(model.getField());
                }
            }
            sql.append(" WHERE ").append(array.get(0).getName()).append(".").append(array.get(0).getField());
            sql.append(" = '").append(agencyCode).append("' ;");
        }
        return sql;
    }

    /**
     *
     * @param connection
     * @param agencyCode
     * @param dbName
     * @return
     * @throws SQLException
     */

    public String insertData(Connection connection, String agencyCode, String dbName) throws SQLException {
        try {
            if (!connection.isClosed())
                System.out.println("Succeeded connecting to the Database!");
            //通过连接获取relation表树形结构
            List<RelationTreeModel> trees = this.createRelationTree(dbName);
            for (RelationTreeModel tree : trees) {
                //根据根节点获得根节点相关列表
                List<RelationTreeModel> array = this.createList(tree);
                //根据列表获取sql语句
                for(RelationTreeModel model: array){
                    List<RelationTreeModel> tableSet = tableRelation(model, array);
                    List<String> columNames = this.selectColumNames(model.getName(),connection);
                    String sql = this.insertSqlByTableName(tableSet , model.getName(), columNames, agencyCode ).toString();
                    System.out.println(sql);
                    String columnSql = "" +
                            "\t\t SELECT\n" +
                            "\t\t table_name as table_name,\n" +
                            "\t\t\tcolumn_name as columnName,\n" +
                            "\t\t\t( CASE WHEN ( is_nullable = 'no' && column_key != 'PRI' ) THEN '1' ELSE NULL END ) AS isRequired,\n" +
                            "\t\t\t( CASE WHEN column_key = 'PRI' THEN '1' ELSE '0' END ) AS isPk,\n" +
                            "\t\t\tordinal_position AS sort,\n" +
                            "\t\t\tcolumn_comment as columnComment,\n" +
                            "\t\t\t( CASE WHEN extra = 'auto_increment' THEN '1' ELSE '0' END ) AS isIncrement,\n" +
                            "\t\t\tcolumn_type as columnType \n" +
                            "\t\tFROM\n" +
                            "\t\t\tinformation_schema.COLUMNS \n" +
                            "\t\tWHERE\n" +
                            "\t\t\ttable_schema = database()\n" +
                            "\t\t\tAND table_name = \"" + model.getName() + "\"\n" +
                            "\t\tORDER BY\n" +
                            "\t\t\tordinal_position\t";
                    List<Map<String, Object>> dataList = this.setToMap(sql,connection);
                    List<Map<String, Object>> columnList = this.setToMap(columnSql,connection);
                    List<TableColumn> columns = new ArrayList<>();
                    for (Map<String,Object> map: columnList){
                        TableColumn column = JSON.parseObject(JSON.toJSONString(map), TableColumn.class);
                        columns.add(column);
                    }
                    StringBuffer sbsql = new StringBuffer();
                    for (Map<String, Object> map : dataList) {
                        sbsql.append(INSERT_INTO);
                        sbsql.append(SPACE).append(model.getName()).append(SPACE);
                        sbsql.append(LEFTBRACE);
                        for (TableColumn column : columns) {
                            sbsql.append(SPLIT);
                            sbsql.append(column.getColumnName());
                            sbsql.append(SPLIT);
                            sbsql.append(COMMA);
                        }
                        sbsql.deleteCharAt(sbsql.length() - 1);
                        sbsql.append(RIGHTBRACE);
                        sbsql.append(VALUES);
                        sbsql.append(LEFTBRACE);
                        for (TableColumn column : columns) {
                            if (column.getColumnType().contains("bigint") ||
                                    column.getColumnType().contains("int") ||
                                    column.getColumnType().contains("bit(1)")) {
                                if (!ObjectUtils.isEmpty(map.get(column.getColumnName()))) {
                                    sbsql.append(map.get(column.getColumnName()));
                                }else {
                                    sbsql.append("0");
                                }
                            } else {
                                if (!ObjectUtils.isEmpty(map.get(column.getColumnName()))) {
                                    sbsql.append("'").append(map.get(column.getColumnName())).append("'");
                                } else {
                                    sbsql.append("null");
                                }
                            }
                            sbsql.append(COMMA);
                        }
                        sbsql.deleteCharAt(sbsql.length() - 1);
                        sbsql.append(RIGHTBRACE);
                        sbsql.append(BRANCH).append(BR);
                    }
                     download(sql);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        connection.close();
        return null;
    }
    /**
     * 导出插入语句
     * @param array
     * @param agencyCode
     * @return
     */
    public StringBuilder insertSqlByTableName(
            List<RelationTreeModel> array,
            String tableName,
            List<String> columNames,
            String agencyCode) {
        StringBuilder sql = new StringBuilder();
        if (array.size() == 1) {
            sql.append("SELECT * FROM ").append(array.get(0).getName()).append(" WHERE ");
            sql.append(array.get(0).getName()).append(".").append(array.get(0).getField());
            sql.append(" = '").append(agencyCode).append("' ;");
        } else {
            sql.append("SELECT ");
            int count = 0;
            for(String name : columNames){
                if (count != 0){
                    sql.append(",");
                }
                count++;
                sql.append(tableName).append(".").append(name);
            }
            sql.append(" FROM ").append(array.get(0).getName());
            for (RelationTreeModel model : array) {
                if (!model.getParentName().equals("")) {
                    sql.append(" JOIN ").append(model.getName()).append(" ON ");
                    sql.append(model.getParentName()).append(".").append(model.getParentField()).append(" = ");
                    sql.append(model.getName()).append(".").append(model.getField());
                }
            }
            sql.append(" WHERE ").append(array.get(0).getName()).append(".").append(array.get(0).getField());
            sql.append(" = '").append(agencyCode).append("' ;");
        }
        return sql;
    }

    /**
     *
     * @param tableName
     * @param connection
     * @return
     * @throws SQLException
     */
    public List<String> selectColumNames(String tableName, Connection connection) throws SQLException {
        List<String> columNames = new ArrayList<>();
        String sql = "SELECT * FROM " + tableName + " limit 0,0";
        Statement statement = connection.createStatement();
        ResultSet resultSet = statement.executeQuery(sql);
        ResultSetMetaData data = resultSet.getMetaData();
        for(int i = 1 ; i <= data.getColumnCount(); i++ ){
            if(!("content").equals(data.getColumnName(i)))
                columNames.add(data.getColumnName(i));
        }
        return columNames;
    }
    /**
     * 列表节点生成（递归）
     */
    public List<RelationTreeModel> createNodeList(List<RelationTreeModel> tree) {
        List<RelationTreeModel> result = new ArrayList<>();
        for (RelationTreeModel node : tree) {
            RelationTreeModel model = new RelationTreeModel();
            BeanUtils.copyProperties(node, model);
            model.setChildren(null);
            result.add(model);
            result.addAll(createNodeList(node.getChildren()));
        }
        return result;
    }

    /**
     * 树转化对象列表
     */
    public List<RelationTreeModel> createList(RelationTreeModel model) {
        List<RelationTreeModel> result = new ArrayList<>();
        RelationTreeModel temp = new RelationTreeModel();
        BeanUtils.copyProperties(model, temp);
        temp.setChildren(null);
        result.add(temp);
        result.addAll(createNodeList(model.getChildren()));
        return result;
    }

    /**
     *
     * @param table
     * @param list
     * @return
     */
    public List<RelationTreeModel>  tableRelation(RelationTreeModel table, List<RelationTreeModel> list){
        List<RelationTreeModel> result = new ArrayList<>();
        if(table.getParentName().equals("")) {
            result.add(table);
        }
        else {
            for (RelationTreeModel node : list) {
                if (node.getName().equals(table.getParentName())) {
                    result.addAll(tableRelation(node, list));
                    result.add(table);
                    break;
                }
            }
        }
        return result;
    }
    /**
     * 获取关系表转化为map
     */
    public List<Map<String, Object>> setToMap(String sql, Connection connection) throws SQLException {
        Statement statement = connection.createStatement();
        ResultSet rs = statement.executeQuery(sql);
        List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
        try {
            ResultSetMetaData md = rs.getMetaData();
            int columnCount = md.getColumnCount();
            while (rs.next()) {
                Map<String, Object> rowData = new HashMap<String, Object>();
                for (int i = 1; i <= columnCount; i++) {
                    rowData.put(md.getColumnName(i), rs.getObject(i));
                }
                list.add(rowData);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            try {
                if (rs != null)
                    rs.close();
                rs = null;
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        return list;
    }

    public void outputAgency(Long agencyCodeId, String sequenceNbrs) throws Exception {
        for (Long seq : (Long[]) ConvertUtils.convert(StringUtil.string2Array(sequenceNbrs),Long.class)) {
            DataBaseLinkModel model = dataBaseLinkService.queryBySeq(seq);
            Connection connection = dataBaseLinkService.connectNewDatabase(model);
            this.insertData(
                    connection, agencyTreeService.queryForAgencyCodeById(agencyCodeId), model.getDbName());
        }
    }


    public void download(String filePath ,String sql) throws IOException {
        File file = new File(filePath);
        try {
            FileWriter writer = new FileWriter(file);
            writer.write(sql);
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
